import java.io.*;
import java.util.*;

/**
 * <h1>FittsTwoThumb</h1>
 * 
 * <h3>Summary</h3>
 * 
 * <ul>
 * <li>
 * A program to predict the asymptotic text entry rate for two-thumb input on a miniature keyboard
 * <p>
 * 
 * <li>Predictions available for any layout (including QWERTY)
 * <p>
 * </ul>
 * 
 * <h3>Background</h3>
 * 
 * Text entry using two thumbs on a miniature Qwerty keyboard is distinctly different from touch
 * typing on a standard keyboard or stylus input on a soft keyboard. The full details of the
 * two-thumb text entry model are provided in the following paper:
 * <p>
 * 
 * <ul>
 * <li>MacKenzie, I. S., & Soukoreff, R. W. (2002). <a href="http://www.yorku.ca/mack/gi2002.html">A
 * model of two-thumb text entry</a>. <i>Proceedings of Graphics Interface 2002 - GI 2002</i>, pp.
 * 117-124. Toronto: Canadian Information Processing Society.
 * </ul>
 * <p>
 * 
 * <h3>Parameters</h3>
 * 
 * This program works with parameters contained in three text files. The first is <a
 * href="FittsTwoThumb.txt">FittsTwoThumb.txt</a>. The contents specify various components and
 * parameters for the model.
 * <p>
 * 
 * The nominal parameter settings are...
 * <p>
 * 
 * <pre>
*     word-frequency file = d1-wordfreq.txt
*     keyboard definition file = SharpEL-6810.txt
*     left thumb letters = qwertasdfgzxcvb
*     right thumb letters = yuiophjklnm
*     left intercept = 176 ms
*     left slope = 64 ms/bit
*     right intercept = 176 ms
*     right slope = 64 ms/bit
*     tMin = 0.088 s
*     space key policy = Alternate
* </pre>
 * 
 * Lines beginning with "#" are ignored. Edit <code>FittsTwoThumb.txt</code>, as necessary, to
 * change the parameter settings.
 * <p>
 * 
 * The other two required text files are specified in the first two lines of
 * <code>FittsTwoThumb.txt</code>. The first line in <code>FittsTwoThumb.txt</code> specifies a
 * word-frequency file. This file is a reduced version of a corpus. It contains the unique words in
 * the corpus and the frequency of each word. Examples appearing in <code>FittsTwoThumb.txt</code>
 * include
 * <p>
 * 
 * <ul>
 * <li><a href="d1-wordfreq.txt">d1-wordfreq.txt</a> &ndash; 9022 unique words
 * <li><a href="d2-wordfreq.txt">d2-wordfreq.txt</a> &ndash; 64,566 unique words
 * <li><a href="bc-wordfreq.txt">bc-wordfreq.txt</a> &ndash; 41,532 unique words
 * <li><a href="phrases2-wordfreq1.txt">phrases2-wordfreq1.txt</a> &ndash; 1164 unique words
 * <li><a href="hello.txt">hello.txt</a> &ndash; 2 unique words (for debugging)
 * </ul>
 * <p>
 * 
 * The second line in <code>FittsTwoThumb.txt</code> specifies a keyboard definition file. This file
 * defines the keyboard. Each line contains four space-delimited entries: (i) a character, (ii)
 * digitized <i>x</i>-position of key, (iii) digitized <i>y</i>-position of key, and (iv) width of
 * key. Examples appearing in <code>FittsTwoThumb.txt</code> include
 * <p>
 * 
 * <ul>
 * <li><a href="TwoType.txt">TwoType.txt</a>
 * <li><a href="SharpEL-6810.txt">SharpEL-6810.txt</a>
 * (<a href="SharpEL-6810.jpg">image</a>)
 * <li><a href="SharpEL-6810-10.txt">SharpEL-6810-10.txt</a>
 * <li><a href="SharpEL-6810-20.txt">SharpEL-6810-20.txt</a>
 * <li><a href="SharpEL-6810-50.txt">SharpEL-6810-50.txt</a>
 * <li><a href="Nokia5510.txt">Nokia5510.txt</a>
 * (<a href="Nokia5510-2.jpg">image</a>)
 * <li><a href="Nokia5510b.txt">Nokia5510b.txt</a>
 * </ul>
 * <p>
 * 
 * <h3>Operation</h3>
 * 
 * Example invocations:
 * <p>
 * 
 * <pre>
*     PROMPT>java FittsTwoThumb ?
*     usage: java FittsTwoThumb [-p] [-b] [-d] [-m] [-t]
*     
*     Output options: (default is no output)
*        -p = prediction
*        -b = breakdown of prediction
*        -d = debug information
*        -m = model components and parameters
*        -t = thumb usage statistics
*     
*     Note: Model definition is read from FittsTwoThumb.txt
*     
*     PROMPT>java FittsTwoThumb -p
*     wpm = 59.18646136479296
*     
*     PROMPT>java FittsTwoThumb -b
*     Number of unique words = 9022
*     Total of frequencies = 67962112
*     Quickest to enter = a (0.267 seconds)
*     Slowest to enter = telecommunications (4.319 seconds)
*     tCorpus = 7.478035148478729E7
*     nCorpus = 368832032
*     tChar = 0.20274907002867712
*     -----
*     
*     PROMPT>java FittsTwoThumb -m
*     MODEL PARAMETERS
*     Corpus: d1-wordfreq.txt
*     Keyboard: SharpEL-6810
*     Left thumb letter assignments: qwertasdfgzxcvb
*     Right thumb letter assignments: yuiophjklnm
*     Left thumb ratio of word endings: 0.7048896155552081
*     Right thumb ratio of word endings: 0.2951103844447918
*     Fitts' law coefficients...
*     Left thumb intercept: 176.0
*     Left thumb slope: 64.0
*     Right thumb intercept: 176.0
*     Right thumb slope: 64.0
*     tMIN: 0.088
*     Space key policy: Alternate
*     -----
*     
*     PROMPT>java FittsTwoThumb -t
*     THUMB USAGE
*     All... (n = 368832032)
*     L = 195459980 (52.99%), R = 173372052 (47.01%)
*     LL = 78917833 (21.40%), LR = 116542147 (31.60%), RR = 56829905 (15.41%), RL = 116542147 (31.60%)
*     Word beginnings... (n = 67962112)
*     L = 44686347 (65.75%), R = 23275765 (34.25%)
*     Word endings... (n = 67962112)
*     L = 47905787 (70.49%), R = 20056325 (29.51%)
*     -----
*     
*     PROMPT>type hello.txt
*     hello 1
*     
*     PROMPT>java FittsTwoThumb -d      (Note: word-freq file = hello.txt)
*     *** hello ***
*     1 : h : .R     : 0.306 -> 1.0*tFitts + 0.0*tMin
*     2 : e : RL <0> : 0.088 -> max(0.306+0.088, 0.000+0.377)
*     3 : l : LR <1> : 0.271 -> max(0.394+0.088, 0.306+0.359)
*     4 : l : RR     : 0.176 -> tFitts
*     5 : o : RR     : 0.270 -> tFitts
*     6 : _ : RL <2> : 0.088 -> max(1.112+0.088, 0.394+0.377)
*     t[] = 0.306, 0.394, 0.666, 0.842, 1.112, 1.200
*     -----
* </pre>
 * 
 * @author Scott MacKenzie, 2001-2015
 */
public class FittsTwoThumb
{
	public static void main(String[] args) throws IOException
	{
		int i;

		if (args.length == 1 && args[0].equals("?"))
		{
			usage();
			return;
		}

		boolean predictionOption = false;
		boolean breakdownOption = false;
		boolean debugOption = false;
		boolean modelOption = false;
		boolean thumbOption = false;
		for (i = 0; i < args.length; ++i)
		{
			if (args[i].equals("-p"))
				predictionOption = true;
			if (args[i].equals("-b"))
				breakdownOption = true;
			if (args[i].equals("-d"))
				debugOption = true;
			if (args[i].equals("-m"))
				modelOption = true;
			if (args[i].equals("-t"))
				thumbOption = true;
		}

		int wordCount = 0;
		int totalFreq = 0;

		BufferedReader modelDef = new BufferedReader(new FileReader("FittsTwoThumb.txt"));

		String line;
		while ((line = modelDef.readLine()).charAt(0) == '#')
			; // skip comments
		String corpus = line;

		while ((line = modelDef.readLine()).charAt(0) == '#')
			; // skip comments

		// read name of Keyboard Definition file

		i = line.indexOf(".txt");
		String keyboard;
		if (i == -1)
			keyboard = line;
		else
			keyboard = line.substring(0, i);

		BufferedReader keydefFile = new BufferedReader(new FileReader(line));
		while ((line = modelDef.readLine()).charAt(0) == '#')
			;

		String leftThumb = line;
		while ((line = modelDef.readLine()).charAt(0) == '#')
			;

		String rightThumb = line;
		while ((line = modelDef.readLine()).charAt(0) == '#')
			;

		double leftIntercept = Double.parseDouble(line);
		while ((line = modelDef.readLine()).charAt(0) == '#')
			;

		double leftSlope = Double.parseDouble(line);
		while ((line = modelDef.readLine()).charAt(0) == '#')
			;

		double rightIntercept = Double.parseDouble(line);
		while ((line = modelDef.readLine()).charAt(0) == '#')
			;

		double rightSlope = Double.parseDouble(line);
		while ((line = modelDef.readLine()).charAt(0) == '#')
			;

		double tMin = Double.parseDouble(line);
		while ((line = modelDef.readLine()).charAt(0) == '#')
			;

		String spaceKeyPolicy = line;

		// end of Keyboard Definition file

		StringTokenizer st;
		Key[] k = new Key[100];
		int kCount = 0;

		// read the keyboard definition file

		while ((line = keydefFile.readLine()) != null)
		{
			st = new StringTokenizer(line);
			if (st.countTokens() != 4)
			{
				System.out.println("data format error!");
				System.exit(1);
			}
			String s;
			double d1, d2, d3;
			s = st.nextToken();
			d1 = Double.parseDouble(st.nextToken());
			d2 = Double.parseDouble(st.nextToken());
			d3 = Double.parseDouble(st.nextToken());
			k[kCount] = new Key(s, d1, d2, d3);
			++kCount;
		}

		double tCorpus = 0.0; // total time to enter corpus
		long nCorpus = 0; // number of characters in corpus
		double tmp = 0.0;

		// counts of thumb usage

		long countL = 0; // left
		long countR = 0; // right
		long countLL = 0; // left-left
		long countRR = 0; // right-right
		long countLR = 0; // left-right
		long countRL = 0; // right-left
		long firstL = 0; // first letter in word = left thumb
		long firstR = 0;
		long lastL = 0;
		long lastR = 0;

		// determine words that are the quickest and slowest to enter

		String quickestWord = "";
		double quickestTime = Double.MAX_VALUE;
		String slowestWord = "";
		double slowestTime = 0.0;

		// open the word-freq file (we need to read it twice)

		BufferedReader ksFile = new BufferedReader(new FileReader(corpus));

		// gather some preliminary stats (1st time through)

		while ((line = ksFile.readLine()) != null)
		{
			++wordCount;

			st = new StringTokenizer(line);
			if (st.countTokens() != 2)
			{
				System.out.println("data format error!");
				System.exit(1);
			}

			// get the word
			String word = st.nextToken();

			// get the frequency of the word
			int freq = Integer.parseInt(st.nextToken());

			totalFreq += freq;

			String first = word.substring(0, 1);
			String last = word.substring(word.length() - 1, word.length());
			if (leftThumb.indexOf(first) >= 0)
				firstL += freq;
			if (rightThumb.indexOf(first) >= 0)
				firstR += freq;
			if (leftThumb.indexOf(last) >= 0)
				lastL += freq;
			if (rightThumb.indexOf(last) >= 0)
				lastR += freq;

		}
		ksFile.close();
		double leftRatio = (double)lastL / totalFreq;
		double rightRatio = (double)lastR / totalFreq;

		// reopen the word+freq file

		ksFile = new BufferedReader(new FileReader(corpus));

		while ((line = ksFile.readLine()) != null)
		{
			st = new StringTokenizer(line);
			if (st.countTokens() != 2)
			{
				System.out.println("data format error!");
				System.exit(1);
			}

			// get the word
			String word = st.nextToken();

			// get the frequency of the word
			int freq = Integer.parseInt(st.nextToken());

			// get the required keystroke sequence
			String ks = "_" + word + "_"; // space at each end

			if (debugOption)
				System.out.println("\n*** " + word + " ***");

			// double mt = 0.0; // movement time
			double t[] = new double[50]; // array of times, one per letter

			// analyse the word, moving through by letter pairs

			for (int idx = 1; idx < ks.length(); ++idx)
			{
				String char1 = ks.substring(idx - 1, idx);
				String char2 = ks.substring(idx, idx + 1);

				String thumb1;
				String thumb2;

				if (leftThumb.indexOf(char1) >= 0)
					thumb1 = "L";
				else if (rightThumb.indexOf(char1) >= 0)
					thumb1 = "R";
				else
					thumb1 = "."; // unassigned

				if (leftThumb.indexOf(char2) >= 0)
					thumb2 = "L";
				else if (rightThumb.indexOf(char2) >= 0)
					thumb2 = "R";
				else
					thumb2 = "."; // unassigned

				// special treatment for preceding SPACE

				if (idx == 1)
				{
					if (spaceKeyPolicy.equals("Alternate"))
						thumb1 = "."; // don't know which thumb is used

					if (spaceKeyPolicy.equals("Left"))
						thumb1 = "L";

					if (spaceKeyPolicy.equals("Right"))
						thumb1 = "R";
				}

				// special treatment for terminating SPACE

				if (idx == ks.length() - 1)
				{
					// Depends on 'Space Key Policy'

					if (spaceKeyPolicy.equals("Alternate"))
						if (thumb1.equals("L"))
							thumb2 = "R";
						else
							thumb2 = "L";

					if (spaceKeyPolicy.equals("Left"))
						thumb2 = "L";

					if (spaceKeyPolicy.equals("Right"))
						thumb2 = "R";
				}

				// determine key distance and width (for Fitts' law)
				double dist = getDistance(k, kCount, char1, char2);
				double width = getWidth(k, kCount, char2);

				if (thumb2.equals("L"))
					countL += freq;
				if (thumb2.equals("R"))
					countR += freq;
				if (thumb1.equals("L") && thumb2.equals("L"))
					countLL += freq;
				if (thumb1.equals("R") && thumb2.equals("R"))
					countRR += freq;
				if (thumb1.equals("L") && thumb2.equals("R"))
					countLR += freq;
				if (thumb1.equals("R") && thumb2.equals("L"))
					countRL += freq;

				// OK, we're ready to compute the entry time...

				// 1st character gets special treatment
				if (idx == 1)
				{
					// word begins with Left thumb

					if (thumb2.equals("L"))
					{
						// again, depends on 'Space Key Policy'

						if (spaceKeyPolicy.equals("Alternate"))
						{
							tmp = rightRatio * fitts(dist, width, leftIntercept, leftSlope) + leftRatio * tMin;
							if (debugOption)
								System.out.printf("1 : %s : %s     : %5.3f -> %f*tFitts + %f*tMin\n", char2,
										(thumb1 + thumb2), tmp, rightRatio, leftRatio);
						} else if (spaceKeyPolicy.equals("Left"))
						{
							tmp = fitts(dist, width, leftIntercept, leftSlope);
							if (debugOption)
								System.out.printf("1 : %s : %s     : %5.3f -> tFitts\n", char2, (thumb1 + thumb2), tmp);
						} else if (spaceKeyPolicy.equals("Right"))
						{
							tmp = tMin;
							if (debugOption)
								System.out.printf("1 : %s : %s     : %5.3f -> tMin\n", char2, (thumb1 + thumb2), tmp);
						}
					} else

					// word begins with Right thumb
					{
						// depends on 'Space Key Policy'

						if (spaceKeyPolicy.equals("Alternate"))
						{
							tmp = leftRatio * fitts(dist, width, rightIntercept, rightSlope) + rightRatio * tMin;
							if (debugOption)
								System.out.printf("1 : %s : %s     : %5.3f -> %f*tFitts + %f*tMin\n", char2,
										(thumb1 + thumb2), tmp, leftRatio, rightRatio);
						} else if (spaceKeyPolicy.equals("Right"))
						{
							tmp = fitts(dist, width, rightIntercept, rightSlope);
							if (debugOption)
								System.out.printf("1 : %s : %s     : %5.3f -> tFitts\n", char2, (thumb1 + thumb2), tmp);
						} else if (spaceKeyPolicy.equals("Left"))
						{
							tmp = tMin;
							if (debugOption)
								System.out
										.printf("1 : %s : %s     : %5.3f -> -> tMin\n", char2, (thumb1 + thumb2), tmp);
						}
					}
					t[1] = tmp; // time stamp for 1st letter in word
				} else
				{
					if (thumb1.equals(thumb2))
					{
						if (thumb1.equals("L")) // Left-Left
						{
							tmp = fitts(dist, width, leftIntercept, leftSlope);
							if (debugOption)
								System.out.printf("%d : %s : %s     : %5.3f -> tFitts\n", idx, char2,
										(thumb1 + thumb2), tmp);
						} else
						// Right-Right
						{
							tmp = fitts(dist, width, rightIntercept, rightSlope);
							if (debugOption)
								System.out.printf("%d : %s : %s     : %5.3f -> tFitts\n", idx, char2,
										(thumb1 + thumb2), tmp);
						}
						t[idx] = t[idx - 1] + tmp; // time stamp (if LL or RR)
					} else
					// thumb1 != thumb2
					{
						// double tRecent = 0.0;
						int recent = 0;
						if (thumb2.equals("R")) // Left-Right
						{
							// find index of most recent use of right thumb
							for (recent = idx - 1; recent > 0; --recent)
								if (rightThumb.indexOf(ks.substring(recent, recent + 1)) >= 0)
									break;

							// recompute 'dist' for new char1
							dist = getDistance(k, kCount, ks.substring(recent, recent + 1), char2);

							t[idx] = Math.max(t[idx - 1] + tMin, t[recent]
									+ fitts(dist, width, rightIntercept, rightSlope));

							if (debugOption)
							{
								System.out.print(idx + " : " + char2 + " : " + thumb1 + thumb2);
								System.out.print(" <" + recent + "> ");
								System.out.printf(": %5.3f -> max(%5.3f:%5.3f, %5.3f+%5.3f)\n", (t[idx] - t[idx - 1]),
										t[idx - 1], tMin, t[recent], fitts(dist, width, rightIntercept, rightSlope));
							}
						} else
						// must be Right-Left
						{
							// find index of most recent use of left thumb
							for (recent = idx - 1; recent > 0; --recent)
								if (leftThumb.indexOf(ks.substring(recent, recent + 1)) >= 0)
									break;

							// recompute 'dist' for new char1
							dist = getDistance(k, kCount, ks.substring(recent, recent + 1), char2);

							t[idx] = Math.max(t[idx - 1] + tMin, t[recent]
									+ fitts(dist, width, leftIntercept, leftSlope));

							if (debugOption)
							{
								System.out.print(idx + " : " + char2 + " : " + thumb1 + thumb2);
								System.out.print(" <" + recent + "> ");
								System.out.printf(": %5.3f -> max(%5.3f+%5.3f, %5.3f+%5.3f)\n", (t[idx] - t[idx - 1]),
										t[idx - 1], tMin, t[recent], fitts(dist, width, leftIntercept, leftSlope));
							}
						}
					}
				}
			}

			if (debugOption)
			{
				System.out.print("t[] = ");
				for (i = 1; i < ks.length() - 1; ++i)
					System.out.printf("%5.3f, ", t[i]);
				System.out.printf("%5.3f\n", t[i]);
			}

			if (t[ks.length() - 1] > slowestTime)
			{
				slowestWord = word;
				slowestTime = t[ks.length() - 1];
			}

			if (t[ks.length() - 1] < quickestTime)
			{
				quickestWord = word;
				quickestTime = t[ks.length() - 1];
			}

			tCorpus += freq * t[ks.length() - 1];
			nCorpus += freq * (ks.length() - 1); // don't count space at beginning
		}
		if (debugOption)
			System.out.println("-----");

		double tChar = tCorpus / nCorpus; // mean time per character
		if (breakdownOption)
		{
			System.out.println("Number of unique words = " + wordCount);
			System.out.println("Total of frequencies = " + totalFreq);
			System.out.printf("Quickest to enter = %s (%5.3f seconds)\n", quickestWord, quickestTime);
			System.out.printf("Slowest to enter = %s (%5.3f seconds)\n", slowestWord, slowestTime);
			System.out.println("tCorpus = " + tCorpus);
			System.out.println("nCorpus = " + nCorpus);
			System.out.println("tChar = " + tChar);
			System.out.println("-----");
		}

		// an adjustment is needed for SKP = Alternate

		if (spaceKeyPolicy.equals("Alternate"))
		{
			countLL += Math.round((totalFreq * ((double)lastR / totalFreq) * ((double)firstL / totalFreq)));
			countLR += Math.round((totalFreq * ((double)lastR / totalFreq) * ((double)firstR / totalFreq)));
			countRR += Math.round((totalFreq * ((double)lastL / totalFreq) * ((double)firstR / totalFreq)));
			countRL += Math.round((totalFreq * ((double)lastL / totalFreq) * ((double)firstL / totalFreq)));
		}

		if (thumbOption)
		{
			System.out.println("THUMB USAGE");
			System.out.printf("All... (n = %d)\n", nCorpus);
			System.out.printf("L = %d (%5.2f%%)", countL, ((double)countL / nCorpus * 100));
			System.out.printf(", R = %d (%5.2f%%)\n", countR, ((double)countR / nCorpus * 100));
			System.out.printf("LL = %d (%5.3f%%)", countLL, ((double)countLL / nCorpus * 100));
			System.out.printf(", LR = %d (%5.2f%%)", countLR, ((double)countLR / nCorpus * 100));
			System.out.printf(", RR = %d (%5.2f%%)", countRR, ((double)countRR / nCorpus * 100));
			System.out.printf(", RL = %d (%5.2f%%)\n", countRL, ((double)countRL / nCorpus * 100));
			System.out.printf("Word beginnings... (n = %d)\n", totalFreq);
			System.out.printf("L = %d (%5.2f%%)", firstL, ((double)firstL / totalFreq * 100));
			System.out.printf(", R = %d (%5.2f%%)\n", firstR, ((double)firstR / totalFreq * 100));
			System.out.printf("Word endings... (n = %d)\n", totalFreq);
			System.out.printf("L = %d (%5.2f%%)", lastL, (leftRatio * 100));
			System.out.printf(", R = %d (%5.2f%%)\n", lastR, (rightRatio * 100));
			System.out.println("-----");
		}
		double wpm = (1.0 / tChar) * 12.0;

		if (modelOption)
		{
			System.out.println("MODEL PARAMETERS");
			System.out.println("Corpus: " + corpus);
			System.out.println("Keyboard: " + keyboard);
			System.out.println("Left thumb letter assignments: " + leftThumb);
			System.out.println("Right thumb letter assignments: " + rightThumb);
			System.out.println("Left thumb ratio of word endings: " + leftRatio);
			System.out.println("Right thumb ratio of word endings: " + rightRatio);
			System.out.println("Fitts' law coefficients...");
			System.out.println("Left thumb intercept: " + leftIntercept);
			System.out.println("Left thumb slope: " + leftSlope);
			System.out.println("Right thumb intercept: " + rightIntercept);
			System.out.println("Right thumb slope: " + rightSlope);
			System.out.println("tMIN: " + tMin);
			System.out.println("Space key policy: " + spaceKeyPolicy);
			System.out.println("-----");
		}

		if (predictionOption)
			System.out.println("wpm = " + wpm);
	}

	// Return a Fitts' law predicted movement time (in seconds).
	private static double fitts(double d, double w, double a, double b)
	{
		double id = Math.log(d / w + 1) / Math.log(2.0);
		return (a + b * id) / 1000.0;
	}

	// get distance between two keys in a key def file
	private static double getDistance(Key[] ka, int count, String s1, String s2)
	{
		int k1 = 0;
		int k2 = 0;
		for (int i = 0; i < count; ++i)
		{
			if (ka[i].getKey().equals(s1))
				k1 = i;
			if (ka[i].getKey().equals(s2))
				k2 = i;
		}
		return ka[k1].distanceTo(ka[k2]);
	}

	@SuppressWarnings("unused")
	private static double getWidth(Key[] ka, int count, String s2)
	{
		int k2 = 0;
		for (int i = 0; i < count; ++i)
		{
			if (ka[i].getKey().equals(s2))
				k2 = i;
			break;
		}
		return ka[k2].getW();
	}

	private static void usage()
	{
		System.out.println("usage: java FittsTwoThumb [-p] [-b] [-d] [-m] [-t]");
		System.out.println();
		System.out.println("Output options: (default is no output)");
		System.out.println("   -p = prediction");
		System.out.println("   -b = breakdown of prediction");
		System.out.println("   -d = debug information");
		System.out.println("   -m = model components and parameters");
		System.out.println("   -t = thumb usage statistics");
		System.out.println();
		System.out.println("Note: Model definition is read from FittsTwoThumb.txt");
		return;
	}
}
